@@ -15,7 +15,8 @@ CGI::Compress::Gzip - CGI with automatically compressed output
 =head1 DESCRIPTION
 
 [This is still alpha code, but it works in some environments.  See
-README for caveats.]
+README for caveats.  I've received reports that it fails under
+Windows.  Help!]
 
 CGI::Compress::Gzip extends the CGI class to auto-detect whether the
 client browser wants compressed output and, if so and if the script
@@ -50,6 +51,13 @@ filehandle passes data verbatim until it detects the end of the CGI
 header.  At that time, it switches over to Gzip output for the
 remainder of the CGI run.
 
+Note that the Zlib library on which this code is ultimately based
+requires a fileno for the output filehandle.  Where the output
+filehandle is faked (i.e. in mod_perl), we instead use in-memory
+compression.  This is more wasteful of RAM, but it is the only
+solution I've found (and it is one shared by the Apache::* compression
+modules).
+
 =cut
 
 require 5.005_62;
@@ -59,12 +67,12 @@ use Carp;
 use CGI;
 
 our @ISA = qw(CGI);
-our $VERSION = '0.04';
+our $VERSION = '0.10';
 
 # Package globals
 
-my $global_use_compression = 1; # user-settable
-my $global_can_compress = undef; # 1 = yes, 0 = no, undef = don't know yet
+our $global_use_compression = 1; # user-settable
+our $global_can_compress = undef; # 1 = yes, 0 = no, undef = don't know yet
 
 #==============================
 
@@ -88,7 +96,7 @@ sub new
 {
    my $pkg = shift;
 
-   $my::Zlibwrapper::use_fh = undef;
+   $CGI::Compress::Gzip::wrapper::use_fh = undef;
    select STDOUT;
    my $self = $pkg->SUPER::new(@_);
    return $self;
@@ -139,7 +147,7 @@ sub useFileHandle
    my $self = shift;
    my $fh = shift;
 
-   $my::Zlibwrapper::use_fh = $fh;
+   $CGI::Compress::Gzip::wrapper::use_fh = $fh;
    return $self;
 }
 #==============================
@@ -193,20 +201,19 @@ sub _canCompress
 
    my $compress = 1;
 
-   # Doesn't work in mod_perl yet
-   $compress &&= !$ENV{MOD_PERL};
-   $compress &&= ((!$ENV{GATEWAY_INTERFACE}) || 
-                  $ENV{GATEWAY_INTERFACE} !~ /^CGI-Perl\//);
-
    # Check programmer preference
    $compress &&= $global_use_compression;
+   #print STDERR "global\n" if ($compress);
 
    # Check that browser supports gzip
    my $acc = $ENV{HTTP_ACCEPT_ENCODING};
    $compress &&= ($acc && $acc =~ /\bgzip\b/i);
+   #print STDERR "accept\n" if ($compress);
 
    # Check that the output will be HTML
    $compress &&= $header && ref($header);
+   #print STDERR "header\n" if ($compress);
+
    if ($compress)
    {
       my $encodingIndex = undef;
@@ -251,6 +258,7 @@ sub _canCompress
          }
       }
    }
+   #print STDERR "header2\n" if ($compress);
 
    # Check that IO::Zlib is available
    if ($compress)
@@ -263,6 +271,7 @@ sub _canCompress
       }
       $compress &&= $global_can_compress;
    }
+   #print STDERR "IO::Zlib\n" if ($compress);
 
    return $compress;
 }
@@ -272,9 +281,9 @@ sub _startCompression
    my $self = shift;
    my $header = shift;
 
-   $my::Zlibwrapper::use_fh ||= \*STDOUT;
+   $CGI::Compress::Gzip::wrapper::use_fh ||= \*STDOUT;
    
-   my $filehandle = my::Zlibwrapper->new($my::Zlibwrapper::use_fh, "wb");
+   my $filehandle = CGI::Compress::Gzip::wrapper->new($CGI::Compress::Gzip::wrapper::use_fh, "wb");
    if (!$filehandle)
    {
       carp "Failed to open Zlib output, reverting to uncompressed output";
@@ -311,19 +320,20 @@ sub DESTROY
 }
 #==============================
 
-package my::Zlibwrapper;
+package CGI::Compress::Gzip::wrapper;
 
 =back
 
 =head1 HELPER CLASS
 
-CGI::Compress::Gzip also implements a helper class in package my::Zlibwrapper
-which subclasses IO::Zlib.  This helper is needed to make sure that
-output is not compressed until the CGI header is emitted.  This
-wrappers delays the ignition of the zlib filter until it sees the
-exact same header generated by CGI::Compress::Gzip::header() pass through it's
-WRITE() method.  If you change the header before printing it, this
-class will throw an exception.
+CGI::Compress::Gzip also implements a helper class in package
+CGI::Compress::Gzip::wrapper which subclasses IO::Zlib.  This helper
+is needed to make sure that output is not compressed until the CGI
+header is emitted.  This wrappers delays the ignition of the zlib
+filter until it sees the exact same header generated by
+CGI::Compress::Gzip::header() pass through it's WRITE() method.  If
+you change the header before printing it, this class will throw an
+exception.
 
 This class hold one global variable representing the previous default
 filehandle used before the gzip filter is put in place.  This
@@ -343,6 +353,8 @@ sub OPEN
 
    # Delay opening until after the header is printed.
    $self->{openargs} = [@_];
+   $self->{outtype} = undef;
+   $self->{buffer} = "";
    return $self;
 }
 
@@ -382,15 +394,6 @@ sub WRITE
          {
             &Carp::croak("Failed to print the uncompressed CGI header");
          }
-         if (!$self->{pending_header})
-         {
-            # Finished printing header!
-            # Complete delayed open
-            if (!$self->SUPER::OPEN(@{$self->{openargs}}))
-            {
-               &Carp::croak("Failed to open the compressed output stream");
-            }
-         }
       }
       else
       {
@@ -399,7 +402,43 @@ sub WRITE
    }
    if ($length)
    {
-      $bytes += $self->SUPER::WRITE($buf, $length, $offset);
+      if (!defined $self->{outtype})
+      {
+         my $fh = $self->{openargs}->[0];
+         my $mod_perl = ($ENV{MOD_PERL} ||
+                         ($ENV{GATEWAY_INTERFACE} &&
+                          $ENV{GATEWAY_INTERFACE} =~ /^CGI-Perl\//));
+         my $isglob = (ref($fh) && ref($fh) eq "GLOB" &&
+                       defined $fh->fileno());
+         my $isfilehandle = (ref($fh) && 
+                             ref($fh) !~ /^GLOB|SCALAR|HASH|ARRAY|CODE$/ &&
+                             $fh->can("fileno") &&
+                             defined $fh->fileno());
+
+         if ((!$mod_perl) && ($isglob || $isfilehandle)) 
+         {
+            # Finished printing header!
+            # Complete delayed open
+            if (!$self->SUPER::OPEN(@{$self->{openargs}}))
+            {
+               &Carp::croak("Failed to open the compressed output stream");
+            }
+            $self->{outtype} = "stream";
+         }
+         else
+         {
+            $self->{outtype} = "block";
+         }
+      }
+      if ($self->{outtype} eq "stream")
+      {
+         $bytes += $self->SUPER::WRITE($buf, $length, $offset);
+      }
+      else
+      {
+         $self->{buffer} .= $buf;
+         $bytes += length $buf;
+      }
    }
    return $bytes;
 }
@@ -413,10 +452,19 @@ sub CLOSE
 
    $use_fh = undef;
    select STDOUT;
-   my $result = $self->SUPER::CLOSE();
-   if (!$result)
+   my $result;
+   if ($self->{outtype} && $self->{outtype} eq "stream")
    {
-      &Carp::confess("Failed to close gzip $!");
+      $result = $self->SUPER::CLOSE();
+      if (!$result)
+      {
+         &Carp::confess("Failed to close gzip $!");
+      }
+   }
+   else
+   {
+      print STDOUT Compress::Zlib::memGzip($self->{buffer});
+      $result = 1;
    }
    return $result;
 }
@@ -426,11 +474,15 @@ __END__
 
 =head1 TO DO
 
-* Fix in mod_perl, test in FastCGI environments
+* Fix under Windows (MinGW?)  Help!  I'm a Mac/Linux guy.
+
+* test in FastCGI environments
 
 * Handle errors more gracefully in WRITE()
 
-* Use memGzip instead of IO::Zlib when output is not a real file handle
+* Better detection of when output is not a real file handle
+
+* Documentation and cleanup of the helper class
 
 =head1 SEE ALSO
 
@@ -444,5 +496,3 @@ Chris Dolan, Clotho Advanced Media, I<chris@clotho.com>
 =head1 LICENSE
 
 GPLv2, see the COPYING file in this distribution.
-
-=cut
